#include "Property.h"
#include <fstream>
#include <iostream>
#include <vector>
#include <windows.h>

using namespace std;

// "GetRevealedTextLength" function signature
typedef size_t(__stdcall * GetTextLengthFcn)(LPCWSTR);

// "RevealText" function signature
typedef void(__stdcall * RevealTextFcn)(LPCWSTR, LPWSTR, size_t);

HMODULE hModule = NULL;

void InitSecurePropSupport(GetTextLengthFcn & aGetTextLengthFcn, RevealTextFcn & aRevealTextFcn);
void LoadPropertiesFromIni(vector<Property> & aProperties);

int main()
{
  //------------------------------------------------------------------------------------------
  // Load the SecureProp library and initialize the exported functions
  //------------------------------------------------------------------------------------------
  GetTextLengthFcn GetRevealedTextLength = NULL;
  RevealTextFcn    RevealText            = NULL;

  InitSecurePropSupport(GetRevealedTextLength, RevealText);

  if (GetRevealedTextLength == NULL || RevealText == NULL)
    return -1;

  // Load the properties from the ini file
  vector<Property> properties;
  LoadPropertiesFromIni(properties);

  if (properties.empty())
    return -1;

  //------------------------------------------------------------------------------------------
  // Use the SecureProp library
  //------------------------------------------------------------------------------------------
  for (size_t i = 0; i < properties.size(); i++)
  {
    LPCWSTR propValue = properties[i].mValue.c_str();

    //------------------------------------------------------------------------------------------
    // Exported function call
    // Reveal the secured text and return its length
    //------------------------------------------------------------------------------------------
    size_t textLength = GetRevealedTextLength(propValue);

    // Allocate a buffer of wide characters with the length: textLength + 1
    // We need to add 1 in order to provide space for the null terminator
    wchar_t * textBuffer = new wchar_t[++textLength];

    //------------------------------------------------------------------------------------------
    // Exported function call
    // Reveal the secured text and store the result in the provided buffer
    //------------------------------------------------------------------------------------------
    RevealText(propValue, textBuffer, textLength);

    // Output the revealed text
    wcout << properties[i].mName << L" = " << textBuffer << endl;

    // Delete the text buffer
    delete[] textBuffer;
    textBuffer = NULL;
  }

  // Free the loaded dll file
  ::FreeLibrary(hModule);

  cin.ignore();

  return 0;
}

void InitSecurePropSupport(GetTextLengthFcn & aGetTextLengthFcn, RevealTextFcn & aRevealTextFcn)
{
  // The name of the dll file to load
  const wstring kDllName = L"SecureProp.dll";

  // Try to load the dll file
  hModule = ::LoadLibrary(kDllName.c_str());
  if (!hModule)
  {
    // Couldn't load the dll file, so just output an error message
    wcerr << L"Couldn't load " << kDllName << endl;
    return;
  }

  // Match the calling convention of the "GetRevealedTextLength" exported function
  aGetTextLengthFcn =
    reinterpret_cast<GetTextLengthFcn>(::GetProcAddress(hModule, "GetRevealedTextLength"));
  if (aGetTextLengthFcn == NULL)
  {
    // Function signature mismatch
    wcerr << L"Couldn't match the function signature of GetRevealedTextLength" << endl;
    return;
  }

  // Match the calling convention of the "RevealText" exported function
  aRevealTextFcn = reinterpret_cast<RevealTextFcn>(::GetProcAddress(hModule, "RevealText"));
  if (aRevealTextFcn == NULL)
  {
    // Function signature mismatch
    wcerr << L"Couldn't match the function signature of RevealText" << endl;
    return;
  }
}

void LoadPropertiesFromIni(vector<Property> & aProperties)
{
  const wstring kIniName = L"SecureProp.ini";

  wifstream f(kIniName, wifstream::in);
  if (!f.is_open())
  {
    // Couldn't load the ini file, so just output an error message
    wcerr << L"Couldn't load " << kIniName << endl;
    return;
  }

  wstring line;
  size_t  found;
  while (getline(f, line))
  {
    found = line.find(L'=');
    if (found != wstring::npos)
      aProperties.push_back(Property(line.substr(0, found), line.substr(found + 1)));
  }

  f.close();
}
